/*
   The classes in this file provide another demonstration
   of polymorphism and also a demonstration of casting
   reference variables. It is important to notice that
   we cast reference variables, not objects!

   One of the lessons from this example is that you must
   keep separate a reference variable from the object it
   refers to. The reference variable is not the object,
   it just refers to the object.
*/

class TopLevelClass
{
   public String aMethod()
   {
      return("this aMethod in TopLevelClass");
   }
}//TopLevelClass


class MidLevelClass extends TopLevelClass
{
   // override aMethd() from TopLevelClass
   public String aMethod()
   {
      return("this is aMethod in MidLevelClass");
   }

   // add a new method to MidLevelClass
   public String bMethod()
   {
      return("this is bMethod in MidLevelClass");
   }
}//MidLevelClass


class LowLevelClass extends MidLevelClass
{
   // override aMethd() from MidLevelClass
   public String aMethod()
   {
      return("this is aMethod in LowLevelClass");
   }

   // override bMethd() from MidLevelClass
   public String bMethod()
   {
      return("this is bMethod in LowLevelClass");
   }

   // add a new method to LowLevelClass
   public String cMethod()
   {
      return("this is cMethod in LowLevelClass");
   }
}//LowLevelClass



public class PolymorphismAndCasting
{
   public static void main(String[] args)
   {
      // Some local reference variables.
      TopLevelClass topLevelReference;
      MidLevelClass midLevelReference;
      LowLevelClass lowLevelReference;

      // Create an object.
      topLevelReference = new LowLevelClass();

      // Make a polymorphic call to aMethod().
      System.out.println( topLevelReference.aMethod() );

      // The following call to bMethod() is not allowed (why not?).
      System.out.println( topLevelReference.bMethod() );

      // Here is how we can call bMethod().
      // The next line combines a cast with a polymorphic method call.
      // (What do you think the printout will be?)
      System.out.println( ((MidLevelClass) topLevelReference).bMethod() );

      // Or we could do this to (polymorphically) call bMethod().
      midLevelReference = (MidLevelClass) topLevelReference;
      System.out.println( midLevelReference.bMethod() );

      // Notice that this will not compile (even though midLevelReference
      // is compatible with the object that topLevelReference points to).
      midLevelReference = topLevelReference;

      // If we want to call cMethod() then we need to do the following.
      // Notice that this is not a polymorphic method call (why not?).
      System.out.println( ((LowLevelClass) topLevelReference).cMethod() );

      // Casting of reference variables can be dangerous.
      topLevelReference = new MidLevelClass();
      // The following line can be compiled, but will cause a runtime error.
      lowLevelReference = (LowLevelClass) topLevelReference;

   }//main

}//PolymorphismAndCasting