/*
   This file tests how polymorphism works with overridden and
   overloaded methods.

   When you run this program, notice how the class of an object (not
   the type of the reference variable) determines which class an
   overridden method comes from, but the type of a reference variable
   (not the class of the object) determines which overloaded method is
   used.
*/

public class TestPolymorphism
{
   public static void main(String[] args)
   {
      // A local reference variable.
      TopLevelClass topLevelReference;

      // The next three pairs of lines all use the same reference
      // variable but they create objects of different types.
      // The three calls to overrideMethod() are polymorphic.
      topLevelReference = new TopLevelClass();
      System.out.println( topLevelReference.overrideMethod() );

      topLevelReference = new MidLevelClass();
      System.out.println( topLevelReference.overrideMethod() );

      topLevelReference = new LowLevelClass();
      System.out.println( topLevelReference.overrideMethod() );



      // The next three pairs of lines all use the same reference
      // variable but they create objects of different types.
      // The three calls to overloadMethod() are NOT polymorphic
      // (but the calls to toSting() inside of overloadMethod() are polymorphic).
      topLevelReference = new TopLevelClass();
      System.out.println( topLevelReference.overloadMethod(topLevelReference) );

      topLevelReference = new MidLevelClass();
      System.out.println( topLevelReference.overloadMethod(topLevelReference) );

      topLevelReference = new LowLevelClass();
      System.out.println( topLevelReference.overloadMethod(topLevelReference) );

   }//main

}//TestPolymorphism