/*
    This is a version of the Shape class as an abstract class.
    It makes more sense to have the Shape class as an abstract
    class than as a "concrete" class since a generic shape does
    not know how to compute its area or perimeter. So it is
    better to have those two methods be abstract.

    By making Shape abstract, notice two things. First, it is no
    longer possible to instantiate a Shape object (but you still
    can have a Shape reference variable). This is good, since a
    Shape object doesn't make much sense. Second, a class that
    inherits from Shape is now forced to implement the computeArea()
    and computePerimeter() methods. When Shape was a "concrete"
    class, a subclass of it could accidentally forget to override
    either computeArea() or computePerimeter(), which would leave
    the subclass with a crippled implementation of the method, which
    would be a bad thing.
*/

public abstract class Shape
{
   // these instance variables should be private, not protected
   protected int x;
   protected int y;
   protected int dimension;

   public Shape(int x, int y, int dimension)
   {
      this.x = x;
      this.y = y;
      this.dimension = dimension;
   }

   // The next two methods are abstract.
   public abstract double computeArea();

   public abstract double computePerimeter();

   // A concrete implementation of toString().
   public String toString()
   {
      return "x = " + x + ", y = " + y + ", dimension = " + dimension;
   }

}//Shape
